VERSION 5.00
Begin VB.UserControl ScreenRes 
   ClientHeight    =   3600
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   4800
   InvisibleAtRuntime=   -1  'True
   Picture         =   "ScreenResolution.ctx":0000
   ScaleHeight     =   3600
   ScaleWidth      =   4800
   ToolboxBitmap   =   "ScreenResolution.ctx":0342
End
Attribute VB_Name = "ScreenRes"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
Private Const CCDEVICENAME = 32
Private Const CCFORMNAME = 32
Private Const DM_BITSPERPEL = &H60000
Private Const DM_PELSWIDTH = &H80000
Private Const DM_PELSHEIGHT = &H100000

' Type: DeviceMode (Screen/Driver Properties)
Private Type DEVMODE
    dmDeviceName As String * CCDEVICENAME
    dmSpecVersion As Integer
    dmDriverVersion As Integer
    dmSize As Integer
    dmDriverExtra As Integer
    dmFields As Long
    dmOrientation As Integer
    dmPaperSize As Integer
    dmPaperLength As Integer
    dmPaperWidth As Integer
    dmScale As Integer
    dmCopies As Integer
    dmDefaultSource As Integer
    dmPrintQuality As Integer
    dmColor As Integer
    dmDuplex As Integer
    dmYResolution As Integer
    dmTTOption As Integer
    dmCollate As Integer
    dmFormName As String * CCFORMNAME
    dmUnusedPadding As Integer
    dmBitsPerPel As Integer
    dmPelsWidth As Long
    dmPelsHeight As Long
    dmDisplayFlags As Long
    dmDisplayFrequency As Long
End Type
Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" (ByVal lpszDeviceName As Long, ByVal iModeNum As Long, lpDevMode As Any) As Boolean
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" (lpDevMode As Any, ByVal dwflags As Long) As Long
Public Enum Resolutions: res640x480 = 640: res800x600 = 800: res1024x768 = 1024: End Enum
Public Enum Colors: clr16 = 4: clr256 = 8: clr16Bit = 16: clr24Bit = 24: clr32Bit = 32: End Enum
Public Enum Responses: respFailed = 0: respSuccess = 1: respRestart = 2: End Enum
Dim OldWidth As Single, OldHeight As Single, OldBPP As Integer
Dim m_Resolution As Resolutions
Dim m_ColorCount As Colors
Dim m_Driver As Integer
Dim m_DriverExtra As Integer
Dim m_DeviceName As String
Dim m_Frequency As Long
Dim m_ChangeResponse As Responses

'====================================================
'= The properties as compressed into single lines merely
'=                              to save screen space.                             =
'====================================================
Public Property Get ChangeResponse() As Responses: ChangeResponse = m_ChangeResponse: End Property
Public Property Let ChangeResponse(ByVal New_ChangeResponse As Responses): DoEvents: End Property
Public Property Get ColorCount() As Colors: ColorCount = m_ColorCount: End Property
Public Property Let ColorCount(ByVal New_ColorCount As Colors): m_ColorCount = New_ColorCount: PropertyChanged "ColorCount": SetResolution: End Property
Public Property Get Driver() As Integer: Driver = m_Driver: End Property
Public Property Let Driver(ByVal New_Driver As Integer): DoEvents: End Property
Public Property Get DriverExtra() As Integer: DriverExtra = m_DriverExtra: End Property
Public Property Let DriverExtra(ByVal New_DriverExtra As Integer): DoEvents: End Property
Public Property Get DeviceName() As String: DeviceName = m_DeviceName: End Property
Public Property Let DeviceName(ByVal New_DeviceName As String): DoEvents: End Property
Public Property Get Frequency() As Long: Frequency = m_Frequency: End Property
Public Property Let Frequency(ByVal New_Frequency As Long): DoEvents: End Property
Public Property Get Resolution() As Resolutions: Resolution = m_Resolution: End Property
Public Property Let Resolution(ByVal New_Resolution As Resolutions): m_Resolution = New_Resolution: PropertyChanged "Resolution": SetResolution: End Property

Private Sub UserControl_Initialize()
    GetSettings
End Sub
Private Sub UserControl_InitProperties()
    GetSettings
End Sub
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    m_Resolution = PropBag.ReadProperty("Resolution", 800)
    m_ColorCount = PropBag.ReadProperty("ColorCount", 0)
    m_Driver = PropBag.ReadProperty("Driver", 0)
    m_DriverExtra = PropBag.ReadProperty("DriverExtra", 0)
    m_DeviceName = PropBag.ReadProperty("DeviceName", "")
    m_Frequency = PropBag.ReadProperty("Frequency", 0)
    m_ChangeResponse = PropBag.ReadProperty("ChangeResponse", 1)
End Sub
Private Sub UserControl_Resize(): Width = 275: Height = 275: End Sub        ' Keep the icon small - takes up less form space!
Private Sub UserControl_Show()
    GetSettings     ' Mainly used just within the IDE to refresh the IDE Property list
End Sub
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    Call PropBag.WriteProperty("Resolution", m_Resolution, 800)
    Call PropBag.WriteProperty("ColorCount", m_ColorCount, 16)
    Call PropBag.WriteProperty("Driver", m_Driver, 0)
    Call PropBag.WriteProperty("DriverExtra", m_DriverExtra, 0)
    Call PropBag.WriteProperty("DeviceName", m_DeviceName, "")
    Call PropBag.WriteProperty("Frequency", m_Frequency, 0)
    Call PropBag.WriteProperty("ChangeResponse", m_ChangeResponse, 1)
End Sub
Private Sub GetSettings()
' Refreshes The Property List
Dim DevM As DEVMODE
    Call EnumDisplaySettings(0&, -1, DevM)
    m_Driver = DevM.dmDriverVersion: PropertyChanged "Driver"
    m_DriverExtra = DevM.dmDriverExtra: PropertyChanged "DriverExtra"
    m_DeviceName = DevM.dmDeviceName: PropertyChanged "DeviceName"
    m_Frequency = DevM.dmDisplayFrequency: PropertyChanged "Frequency"
    m_Resolution = DevM.dmPelsWidth: PropertyChanged "Resolution"
    m_ColorCount = DevM.dmBitsPerPel: PropertyChanged "ColorCount"
    Select Case OldWidth: Case 0: OldWidth = DevM.dmPelsWidth: OldHeight = DevM.dmPelsHeight: OldBPP = DevM.dmBitsPerPel: End Select
End Sub
Private Sub SetResolution()
    DoEvents
    On Error GoTo ResErr
    Dim DevM As DEVMODE, I As Integer, ReturnVal As Boolean, RetValue
    Dim NewWidth As Single, NewHeight As Single, NewBPP As Integer
    NewBPP = m_ColorCount
    Select Case m_Resolution
        Case res640x480: NewWidth = 640: NewHeight = 480        ' Set the variables (NewWidth/Height)
        Case res800x600: NewWidth = 800: NewHeight = 600
        Case res1024x768: NewWidth = 1024: NewHeight = 768
    End Select
    I = 0
    Do
        ' Enumerate the settings (From within the registry)
        ReturnVal = EnumDisplaySettings(0&, I, DevM)
        I = I + 1
    Loop Until (ReturnVal = False)
    
    ' Set the new selection properties
    DevM.dmFields = DM_PELSWIDTH Or DM_PELSHEIGHT Or DM_BITSPERPEL
    DevM.dmPelsWidth = NewWidth
    DevM.dmPelsHeight = NewHeight
    DevM.dmBitsPerPel = NewBPP
    
    ' Make the changes
    Call ChangeDisplaySettings(DevM, 1)

    ' Set the message display property which shows the calling app
    ' weather or not the cahnge took place
    If RetValue = vbCancel Then
        DevM.dmPelsWidth = OldWidth
        DevM.dmPelsHeight = OldHeight
        DevM.dmBitsPerPel = OldBPP
        Call ChangeDisplaySettings(DevM, 1)
        m_ChangeResponse = respFailed: PropertyChanged "ChangedResponse"
    Else
        m_ChangeResponse = respSuccess: PropertyChanged "ChangedResponse"
    End If
    Exit Sub
ResErr:
    m_ChangeResponse = respFailed: PropertyChanged "ChangedResponse"
End Sub
